"""
Degeneracy maps
"""

#*****************************************************************************
#       SAGE: System for Algebra and Geometry Experimentation
#
#       Copyright (C) 2005 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import sage.structure.function as function

class DegeneracyMap(function.MatrixFunction):
    def __init__(self,  matrix, domain, codomain, t):
        self.__t = t
        function.MatrixFunction.__init__(self, matrix, domain, codomain, 'degen')

    def __repr__(self):
        if self.__t == 1:
            pow = ""
        else:
            pow = "^%s"%self.__t
        return "Modular symbols degeneracy map from space of level %s to space of level %s corresponding to f(q) |--> f(q%s)"%(
            self.domain().level(), self.codomain().level(), pow)
    
    def t(self):
        return self.__t

    def image(self):
        V = function.MatrixFunction.image(self)
        if self.domain().is_anemic_hecke_module:
            V._is_anemic_hecke_module = True
        return V

    def kernel(self):
        V = function.MatrixFunction.kernel(self)
        if self.domain().is_anemic_hecke_module:
            V._is_anemic_hecke_module = True
        return V
