"""
Laurent series ring
"""

import weakref

from rational_field import Q
import laurent_series
import power_series_ring
import ring
import sage.structure.gens as gens


_objsLaurentSeriesRing = {}
class _uniqLaurentSeriesRing(object):
    def __new__(cls, base_ring, variable="t"):
        key = (base_ring, variable)
        if _objsLaurentSeriesRing.has_key(key):
            x = _objsLaurentSeriesRing[key]()
            if x != None: return x
        R = object.__new__(cls)
        _objsLaurentSeriesRing[key] = weakref.ref(R)
        return R

class LaurentSeriesRing(_uniqLaurentSeriesRing, ring.Ring, gens.Gens):
    """
    Univariate Laurent Series Ring
    """

    def __init__(self, base_ring, variable="t"):
        self.__variable  = variable
        self.__base_ring = base_ring

    def __repr__(self):
        return "Laurent series ring in %s over %s"%(self.variable(), self.base_ring())
    
    def __call__(self, x, n=0):
        if isinstance(x, laurent_series.LaurentSeries) and n==0 and self == x.parent():
            return x
        return laurent_series.LaurentSeries(self, x, n)
    
    def __cmp__(self, other):
        if not isinstance(other, LaurentSeriesRing):
            return -1
        if self.base_ring() != other.base_ring():
            return -1
        if self.variable() != other.variable():
            return -1
        return 0

    def base_ring(self):
        return self.__base_ring

    def gen(self, n=0):
        if n != 0:
            raise IndexError, "Generator %s not defined."%n
        try:
            return self.__generator
        except AttributeError:
            self.__generator = laurent_series.LaurentSeries(self, [0,1])
            return self.__generator

    def ngens(self):
        return 1

    def variable(self):
        return self.__variable

    def power_series_ring(self):
        """
        If this is the Laurent series ring R((t)), return the power
        series ring R[[t]].
        """
        try:
            return self.__power_series_ring
        except AttributeError:
            self.__power_series_ring = power_series_ring.PowerSeriesRing(
                                         self.base_ring(), self.variable())
            return self.__power_series_ring            
