"""
Univariate Power Series Rings
"""

import weakref
import power_series
import polynomial_ring
import laurent_series_ring
import ring
import sage.structure.gens as gens
from infinity import infinity


_objsPowerSeriesRing = {}
class _uniqPowerSeriesRing(object):
    def __new__(cls, base_ring, variable="t"):
        key = (base_ring, variable)
        if _objsPowerSeriesRing.has_key(key):
            x = _objsPowerSeriesRing[key]()
            if x != None: return x
        R = object.__new__(cls)
        _objsPowerSeriesRing[key] = weakref.ref(R)
        return R

class PowerSeriesRing(_uniqPowerSeriesRing, ring.Ring, gens.Gens):
    def __init__(self, base_ring, variable="t"):
        self.__base_ring = base_ring
        self.__variable = variable
        self.__poly_ring = polynomial_ring.PolynomialRing(base_ring, variable)
        self.__power_series_class = power_series.PowerSeries_generic_dense
        self.__generator = self.__power_series_class(self, [0,1], check=True, is_gen=True)

    def __repr__(self):
        return "Power series ring in %s over %s"%(self.variable(), self.base_ring())

    def __call__(self, f, prec=infinity):
        if isinstance(f, power_series.PowerSeries) and f.parent() == self and prec == f.prec():
            return f
        return self.__power_series_class(self, f, prec)

    def base_ring(self):
        return self.__base_ring

    def _poly_ring(self):
        return self.__poly_ring

    def variable(self):
        return self.__variable

    def gen(self, n=0):
        if n != 0:
            raise IndexError, "Generator %s not defined."%n
        return self.__generator

    def ngens(self):
        return 1

    def random(self, prec, bound=2):
        tmp = self([])
        tmp.randomize(prec,bound)
        return tmp

    def __cmp__(self, other):
        if not isinstance(other, PowerSeriesRing):
            return -1
        if self.base_ring() != other.base_ring():
            return -1
        if self.variable() != other.variable():
            return -1
        return 0

    def __contains__(self, x):
        if not isinstance(x, PowerSeries):
            return False
        return x.parent() == self

    def is_atomic_repr(self):
        return False

    def is_field(self):
        return False

    def is_finite(self):
        return False

    def characteristic(self):
        return self.base_ring().characteristic()

    def name(self):
        return "PowSer%s"%self.base_ring().name()

    def set_variable(self, variable):
        self.__variable = variable

    def laurent_series_ring(self):
        """
        If this is the power series ring R[[t]], return the Laurent
        series ring R((t)).
        """
        try:
            return self.__laurent_series_ring
        except AttributeError:
            self.__laurent_series_ring = laurent_series_ring.LaurentSeriesRing(
                                                 self.base_ring(), self.variable())
            return self.__laurent_series_ring            
    
